@@ -1,4 +1,4 @@
-/*  $Id: inews.c 9366 2011-08-22 08:51:14Z iulius $
+/*  $Id: inews.c 9521 2013-07-31 17:27:33Z iulius $
 **
 **  Send an article (prepared by someone on the local site) to the
 **  master news server.
@@ -462,7 +462,7 @@ FormatUserName(struct passwd *pwp, char *node)
 	    strncpy(out, pwp->pw_name, left);
 	    if (islower((unsigned char) *out)
 	     && (out == outbuff || !isalpha((unsigned char) out[-1])))
-		*out = toupper(*out);
+		*out = toupper((unsigned char) *out);
 	    while (*out) {
 		out++;
                 left--;
