@@ -1,4 +1,4 @@
-/*  $Id: cc.c 9417 2012-06-12 18:31:59Z iulius $
+/*  $Id: cc.c 9584 2013-12-12 20:24:20Z iulius $
 **
 **  Routines for the control channel.
 **
@@ -629,11 +629,14 @@ CCflush(char *av[])
 
 
 /*
-**  Flush the log files.
+**  Flush the log files as well as exploder and process channels.
 */
 static const char *
 CCflushlogs(char *unused[])
 {
+    SITE        *sp;
+    CHANNEL     *cp;
+    int         i;
     unused = unused;		/* ARGSUSED */
 
     if (Debug)
@@ -643,6 +646,15 @@ CCflushlogs(char *unused[])
     syslog(L_NOTICE, "%s flushlogs %s", LogName, CCcurrmode());
     ReopenLog(Log);
     ReopenLog(Errlog);
+    /* Flush exploder and process channels so that they take into account
+     * the new log files (for instance during log rotation). */
+    for (sp = Sites, i = nSites; --i >= 0; sp++) {
+        if (((cp = sp->Channel) != NULL)
+             && ((cp->Type == CTexploder) || (cp->Type == CTprocess))) {
+            SITEflush(sp, true);
+            syslog(L_NOTICE, "%s flush", sp->Name);
+        }
+    }
     return NULL;
 }
 
@@ -940,7 +952,7 @@ CCname(char *av[])
             mode = (cp->MaxCnx > 0 && cp->ActiveCnx == 0) ? "paused" : "";
             buffer_sprintf(&CCreply, true, ":%s:%ld:%s",
                            cp->State == CScancel ? "cancel" : "nntp",
-                           (long) Now.tv_sec - cp->LastActive, mode);
+                           (long) (Now.tv_sec - cp->LastActive), mode);
 	    break;
 	case CTlocalconn:
             buffer_sprintf(&CCreply, true, ":localconn::");
