@@ -1,4 +1,4 @@
-/*  $Id: rc.c 9248 2011-07-17 18:04:50Z iulius $
+/*  $Id: rc.c 9625 2014-03-16 13:24:22Z iulius $
 **
 **  Routines for the remote connect channel.  Create an Internet stream
 **  socket that processes connect to.  If the incoming site is not one of
@@ -32,7 +32,7 @@
 typedef struct _REMOTEHOST {
     char	*Label;         /* Peer label */
     char	*Name;          /* Hostname */
-    struct sockaddr_storage Address;     /* List of ip adresses */
+    struct sockaddr_storage Address;     /* List of ip addresses */
     char	*Password;      /* Optional password */
     char 	*Identd;	/* Optional identd */
     bool	Streaming;      /* Streaming allowed ? */
@@ -1519,7 +1519,7 @@ RCwritelist(char *filename)
         if (*p == '/')
 	   q = p + 1;
 
-    fprintf (F, "##  $Id: rc.c 9248 2011-07-17 18:04:50Z iulius $\n");
+    fprintf (F, "##  $Id: rc.c 9625 2014-03-16 13:24:22Z iulius $\n");
     fprintf (F, "##\n");
     fprintf (F, "##  %s -- Configuration of incoming news feeds\n", q);
     free(r);
