@@ -1,4 +1,4 @@
-/*  $Id: main.c 9275 2011-07-17 19:09:26Z iulius $
+/*  $Id: main.c 9479 2013-06-03 18:12:39Z iulius $
 **
 **  Main routines for the innfeed program.
 **
@@ -572,6 +572,9 @@ static void sighup (int sig UNUSED)
     }
 
   configHosts (talkToSelf) ;
+
+  notice ("ME reloading log file %s", logFile) ;
+  openLogFile() ;
 }
 
 static void sigemt (int sig UNUSED)
