@@ -1,31 +1,37 @@
-/*  $Revision: 1.6.4.1 $
+/*  $Id: clientactive.c 9497 2013-06-25 17:40:59Z iulius $
 **
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include "configdata.h"
+
+#include "config.h"
 #include "clibrary.h"
 #include <errno.h>
-#include "paths.h"
-#include "libinn.h"
-#include "nntp.h"
-#include "macros.h"
+
+#include "inn/innconf.h"
+#include "inn/libinn.h"
+#include "inn/nntp.h"
+#include "inn/paths.h"
 
 
-STATIC char	CApathname[256];
-STATIC FILE	*CAfp;
+static char	*CApathname;
+static FILE	*CAfp;
 
 
 /*
 **  Get a copy of the active file for a client host to use, locally or
 **  remotely.
 */
-FILE *CAopen(FILE *FromServer, FILE *ToServer)
+FILE *
+CAopen(FILE *FromServer, FILE *ToServer)
 {
+    char *path;
+
     /* Use a local (or NFS-mounted) copy if available.  Make sure we don't
      * try to delete it when we close it. */
-    if ((CAfp = fopen(cpcatpath(innconf->pathdb, _PATH_CLIENTACTIVE), "r")) != NULL) {
-	CApathname[0] = '\0';
+    path = concatpath(innconf->pathdb, INN_PATH_CLIENTACTIVE);
+    CAfp = fopen(path, "r");
+    free(path);
+    if (CAfp != NULL) {
+	CApathname = NULL;
 	return CAfp;
     }
 
@@ -37,31 +43,38 @@ FILE *CAopen(FILE *FromServer, FILE *ToServer)
 /*
 **  Internal library routine.
 */
-FILE *CA_listopen(char *pathname, FILE *FromServer, FILE *ToServer, char *request)
+FILE *
+CA_listopen(char *pathname, FILE *FromServer, FILE *ToServer,
+            const char *request)
 {
     char	buff[BUFSIZ];
+    char        expectedanswer[BUFSIZ];
     char	*p;
     int		oerrno;
     FILE	*F;
 
-    (void)unlink(pathname);
-    if ((F = fopen(pathname, "w")) == NULL)
+    F = fopen(pathname, "w");
+    if (F == NULL)
 	return NULL;
 
-    /* Send a LIST command to and capture the output. */
+    /* Send a LIST command and capture the output. */
     if (request == NULL)
-	(void)fprintf(ToServer, "list\r\n");
+	fprintf(ToServer, "LIST\r\n");
     else
-	(void)fprintf(ToServer, "list %s\r\n", request);
-    (void)fflush(ToServer);
+	fprintf(ToServer, "LIST %s\r\n", request);
+    fflush(ToServer);
+
+    snprintf(expectedanswer, sizeof(expectedanswer), "%d", NNTP_OK_LIST);
 
     /* Get the server's reply to our command. */
     if (fgets(buff, sizeof buff, FromServer) == NULL
-     || !EQn(buff, NNTP_LIST_FOLLOWS, STRLEN(NNTP_LIST_FOLLOWS))) {
+     || strncmp(buff, expectedanswer, strlen(expectedanswer)) != 0) {
 	oerrno = errno;
-	/* Only call CAclose() if opened through CAopen() */
-	if (strcmp(CApathname, pathname) == 0) CAclose();
+	/* Only call CAclose() if opened through CAopen(). */
+	if (strcmp(CApathname, pathname) == 0)
+            CAclose();
 	errno = oerrno;
+        fclose(F);
 	return NULL;
     }
 
@@ -76,12 +89,12 @@ FILE *CA_listopen(char *pathname, FILE *FromServer, FILE *ToServer, char *reques
 		break;
 	    return fopen(pathname, "r");
 	}
-	(void)fprintf(F, "%s\n", buff);
+	fprintf(F, "%s\n", buff);
     }
 
     /* Ran out of input before finding the terminator; quit. */
     oerrno = errno;
-    (void)fclose(F);
+    fclose(F);
     CAclose();
     errno = oerrno;
     return NULL;
@@ -92,14 +105,27 @@ FILE *CA_listopen(char *pathname, FILE *FromServer, FILE *ToServer, char *reques
 **  Use the NNTP list command to get a file from a server.  Default is
 **  the active file, otherwise ask for whatever is in the request param.
 */
-FILE *CAlistopen(FILE *FromServer, FILE *ToServer, char *request)
+FILE *
+CAlistopen(FILE *FromServer, FILE *ToServer, const char *request)
 {
-    /* Gotta talk to the server -- see if we can. */
-    if (FromServer == NULL || ToServer == NULL)
-	return NULL;
+    int fd, oerrno;
 
-    (void)sprintf(CApathname, "%.220s/%s", innconf->pathtmp, _PATH_TEMPACTIVE);
-    (void)mktemp(CApathname);
+    /* Gotta talk to the server -- see if we can. */
+    if (FromServer == NULL || ToServer == NULL) {
+	errno = EBADF;
+	return NULL;
+    }
+
+    CApathname = concatpath(innconf->pathtmp, INN_PATH_TEMPACTIVE);
+    fd = mkstemp(CApathname);
+    if (fd < 0) {
+        oerrno = errno;
+        free(CApathname);
+        CApathname = 0;
+        errno = oerrno;
+        return NULL;
+    }
+    close(fd);
     return CAfp = CA_listopen(CApathname, FromServer, ToServer, request);
 }
 
@@ -108,14 +134,15 @@ FILE *CAlistopen(FILE *FromServer, FILE *ToServer, char *request)
 /*
 **  Close the file opened by CAopen or CAlistopen.
 */
-void CAclose(void)
+void
+CAclose(void)
 {
     if (CAfp) {
-	(void)fclose(CAfp);
+	fclose(CAfp);
 	CAfp = NULL;
     }
-    if (CApathname[0]) {
-	(void)unlink(CApathname);
-	CApathname[0] = '\0';
+    if (CApathname != NULL) {
+	unlink(CApathname);
+	CApathname = NULL;
     }
 }
