@@ -1,4 +1,4 @@
-/*  $Id: perl.c 9421 2012-06-12 21:10:35Z iulius $
+/*  $Id: perl.c 9497 2013-06-25 17:40:59Z iulius $
 **
 **  Embedded Perl support for INN.
 **
@@ -52,6 +52,7 @@ HandleHeaders(char *article)
 {
     dSP;
     HEADER *hp;
+    HV *attribs;
     HV *hdr;
     SV *body;
     int rc;
@@ -82,7 +83,16 @@ HandleHeaders(char *article)
     ENTER;
     SAVETMPS;
 
-    /* Create the Perl hash. */
+    /* Create the Perl attributes hash. */
+    attribs = perl_get_hv("attributes", true);
+    (void) hv_store(attribs, "hostname", 8, newSVpv(Client.host, 0), 0);
+    (void) hv_store(attribs, "ipaddress", 9, newSVpv(Client.ip, 0), 0);
+    (void) hv_store(attribs, "port", 4, newSViv(Client.port), 0);
+    (void) hv_store(attribs, "interface", 9, newSVpv(Client.serverhost, 0), 0);
+    (void) hv_store(attribs, "intipaddr", 9, newSVpv(Client.serverip, 0), 0);
+    (void) hv_store(attribs, "intport", 7, newSViv(Client.serverport), 0);
+
+    /* Create the Perl header hash. */
     hdr = perl_get_hv("hdr", true);
     for (hp = Table; hp < EndOfTable; hp++) {
         if (hp->Body)
@@ -177,6 +187,7 @@ HandleHeaders(char *article)
 #endif /* DEBUG_MODIFY */
     }
 
+    hv_undef(attribs);
     hv_undef(hdr);
     sv_setsv(body, &PL_sv_undef);
 
@@ -206,6 +217,11 @@ HandleHeaders(char *article)
 
     if (buf[0] != '\0')
         return buf;
+
+#ifdef DEBUG_MODIFY
+    fclose(flog);
+#endif /* DEBUG_MODIFY */
+
     return NULL;
 }
 
@@ -292,6 +308,7 @@ perlAccess(char *user, struct vector *access_vec)
     }
 
     free(buffer);
+    hv_undef(attribs);
 
     PUTBACK;
     FREETMPS;
