@@ -1,4 +1,4 @@
-/*  $Id: perm.c 9317 2011-08-06 20:24:27Z iulius $
+/*  $Id: perm.c 9603 2014-02-17 13:18:21Z iulius $
 **
 **  How to figure out where a user comes from, and what that user can do once
 **  we know who sie is.
@@ -284,7 +284,11 @@ copy_method(METHOD *orig)
     memset(ConfigBit, '\0', ConfigBitsize);
 
     ret->name = xstrdup(orig->name);
-    ret->program = xstrdup(orig->program);
+    if (orig->program != NULL) {
+        ret->program = xstrdup(orig->program);
+    } else {
+        ret->program = NULL;
+    }
     if (orig->users)
 	ret->users = xstrdup(orig->users);
     else
@@ -716,6 +720,10 @@ authdecl_parse(AUTHGROUP *curauth, CONFFILE *f, CONFTOKEN *tok)
 		tok = CONFgettoken(PERMtoks, f);
 	    }
 
+        if (m->program == NULL) {
+            ReportError(f, "Missing 'program:' key.");
+        }
+
 	    if (tok == NULL) {
 		ReportError(f, "Unexpected EOF.");
 	    }
