@@ -1,46 +1,34 @@
-/* tls.c --- TLSv1 functions
-   Copyright (C) 2000 Kenichi Okada <okada@opaopa.org>
-
-   Author: Kenichi Okada <okada@opaopa.org>
-   Created: 2000-02-22
-
-   Keywords: TLS, OpenSSL
-
-   Commentary:
-
-   [RFC 2246] "The TLS Protocol Version 1.0"
-        by Christopher Allen <callen@certicom.com> and
-        Tim Dierks <tdierks@certicom.com> (1999/01)
-
-   [RFC 2595] "Using TLS with IMAP, POP3 and ACAP"
-        by Chris Newman <chris.newman@innosoft.com> (1999/06)
-
+/*  $Id: tls.c 9261 2011-07-17 18:23:31Z iulius $
+**
+**  tls.c -- TLSv1 functions.
+**  Copyright (C) 2000 Kenichi Okada <okada@opaopa.org>.
+**
+**  Author:  Kenichi Okada <okada@opaopa.org>
+**  Created:  2000-02-22
+**
+**  [RFC 2246] "The TLS Protocol Version 1.0"
+**      by Christopher Allen <callen@certicom.com> and
+**      Tim Dierks <tdierks@certicom.com> (1999/01)
+**
+**  [RFC 2595] "Using TLS with IMAP, POP3 and ACAP"
+**      by Chris Newman <chris.newman@innosoft.com> (1999/06)
 */
 
-#include <sys/types.h>
 #include "config.h"
-#include "nnrpd.h"
-
-#ifdef HAVE_SSL
-
-/* System library. */
-
-#include <unistd.h>
-#include <stdio.h>
-#include <string.h>
+#include "clibrary.h"
 #include <syslog.h>
 #include <sys/stat.h>
 #include <sys/uio.h>
 
-#endif
+#include "nnrpd.h"
+#include "inn/innconf.h"
 
-/* outside the ifdef so `make depend` works even ifndef HAVE_SSL */
+/* Outside the ifdef so that make depend works even ifndef HAVE_SSL. */
 #include "tls.h"
-#include "sasl_config.h"
 
 #ifdef HAVE_SSL
 
-/* We must keep some of the info available */
+/* We must keep some of the info available. */
 static const char hexcodes[] = "0123456789ABCDEF";
 
 static bool tls_initialized = false;
@@ -54,13 +42,13 @@ SSL *tls_conn = NULL;
 #define CCERT_BUFSIZ 256
 
 int     tls_serverengine = 0;
-int     tls_serveractive = 0;	/* available or not */
+int     tls_serveractive = 0;	/* Available or not. */
 char   *tls_peer_subject = NULL;
 char   *tls_peer_issuer = NULL;
 char   *tls_peer_fingerprint = NULL;
 
-int     tls_clientactive = 0;	/* available or not */
-char   *tls_peer_CN = NULL;
+int     tls_clientactive = 0;	/* Available or not. */
+char   *tls_peer_CN= NULL;
 char   *tls_issuer_CN = NULL;
 
 const char   *tls_protocol = NULL;
@@ -72,11 +60,12 @@ int	tls_cipher_algbits = 0;
 int tls_loglevel = 0;
 
 
-/* taken from OpenSSL apps/s_cb.c 
- * tim - this seems to just be giving logging messages
- */
-
-static void apps_ssl_info_callback(SSL * s, int where, int ret)
+/*
+**  Taken from OpenSSL apps/s_cb.c. 
+**  Tim -- this seems to just be giving logging messages.
+*/
+static void
+apps_ssl_info_callback(const SSL *s, int where, int ret)
 {
     const char  *str;
     int         w;
@@ -115,11 +104,11 @@ static void apps_ssl_info_callback(SSL * s, int where, int ret)
 
 
 /*
- *	Hardcoded DH parameter files, from OpenSSL.
- *	For information on how these files were generated, see
- *	"Assigned Number for SKIP Protocols" 
- *	(http://www.skip-vpn.org/spec/numbers.html.
- */
+**  Hardcoded DH parameter files, from OpenSSL.
+**  For information on how these files were generated, see
+**  "Assigned Number for SKIP Protocols" 
+**  <http://www.skip-vpn.org/spec/numbers.html>.
+*/
 static const char file_dh512[] =
 "-----BEGIN DH PARAMETERS-----\n\
 MEYCQQD1Kv884bEpQBgRjXyEpwpy1obEAxnIByl6ypUM2Zafq9AKUJsCRtMIPWak\n\
@@ -158,9 +147,10 @@ AaqLulO7R8Ifa1SwF2DteSGVtgWEN8gDpN3RBmmPTDngyF2DHb5qmpnznwtFKdTL\n\
 KWbuHn491xNO25CQWMtem80uKw+pTnisBRF/454n1Jnhub144YRBoN8CAQI=\n\
 -----END DH PARAMETERS-----\n";
 
+
 /*
- *	Load hardcoded DH parameters.
- */
+**  Load hardcoded DH parameters.
+*/
 static DH *
 load_dh_buffer (const char *buffer, size_t len)
 {
@@ -171,28 +161,30 @@ load_dh_buffer (const char *buffer, size_t len)
 	if (bio == NULL)
 		return NULL;
 	dh = PEM_read_bio_DHparams(bio, NULL, NULL, NULL);
-/*	if (dh == NULL) log error */
+        /* If (dh == NULL) log error? */
 	BIO_free(bio);
 
 	return dh;
 }
 
+
 /*
- *	Generate empheral DH key.  Because this can take a long
- *	time to compute, we use precomputed parameters of the
- *	common key sizes.
- *
- *	These values can be static (once loaded or computed) since
- *	the OpenSSL library can effectively generate random keys
- *	from the information provided.
- *
- *	EDH keying is slightly less efficient than static RSA keying,
- *	but it offers Perfect Forward Secrecy (PFS).
- *
- *	FIXME: support user-specified files, to eliminate risk of
- *	"small group" attacks.
- */
-static DH *tmp_dh_cb(SSL *s UNUSED, int export UNUSED, int keylength)
+**  Generate empheral DH key.  Because this can take a long
+**  time to compute, we use precomputed parameters of the
+**  common key sizes.
+**
+**  These values can be static (once loaded or computed) since
+**  the OpenSSL library can effectively generate random keys
+**  from the information provided.
+**
+**  EDH keying is slightly less efficient than static RSA keying,
+**  but it offers Perfect Forward Secrecy (PFS).
+**
+**  FIXME:  support user-specified files, to eliminate risk of
+**  "small group" attacks.
+*/
+static DH *
+tmp_dh_cb(SSL *s UNUSED, int export UNUSED, int keylength)
 {
 	DH *r = NULL;
 	static DH *dh = NULL;
@@ -224,8 +216,8 @@ static DH *tmp_dh_cb(SSL *s UNUSED, int export UNUSED, int keylength)
 		r = dh4096;
 		break;
 	default:
-		/* we should check current keylength vs. requested keylength */
-		/* also, this is an extremely expensive operation! */
+		/* We should check current keylength vs. requested keylength
+		 * also, this is an extremely expensive operation! */
 		dh = DH_generate_parameters(keylength, DH_GENERATOR_2, NULL, NULL);
 		r = dh;
 	}
@@ -233,9 +225,12 @@ static DH *tmp_dh_cb(SSL *s UNUSED, int export UNUSED, int keylength)
 	return r;
 }
 
-/* taken from OpenSSL apps/s_cb.c */
 
-static int verify_callback(int ok, X509_STORE_CTX * ctx)
+/*
+**  Taken from OpenSSL apps/s_cb.c.
+*/
+static int
+verify_callback(int ok, X509_STORE_CTX * ctx)
 {
     char    buf[256];
     X509   *err_cert;
@@ -248,7 +243,7 @@ static int verify_callback(int ok, X509_STORE_CTX * ctx)
     err = X509_STORE_CTX_get_error(ctx);
     depth = X509_STORE_CTX_get_error_depth(ctx);
 
-    X509_NAME_oneline(X509_get_subject_name(err_cert), buf, 256);
+    X509_NAME_oneline(X509_get_subject_name(err_cert), buf, sizeof(buf));
     if ((tls_serveractive) && (tls_loglevel >= 1))
       Printf("Peer cert verify depth=%d %s", depth, buf);
     if (ok==0)
@@ -257,7 +252,7 @@ static int verify_callback(int ok, X509_STORE_CTX * ctx)
 	     X509_verify_cert_error_string(err));
       
 	if (verify_depth >= depth) {
-	    ok = 0;
+	    ok = 1;
 	    verify_error = X509_V_OK;
 	} else {
 	    ok = 0;
@@ -266,7 +261,7 @@ static int verify_callback(int ok, X509_STORE_CTX * ctx)
     }
     switch (ctx->error) {
     case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT:
-	X509_NAME_oneline(X509_get_issuer_name(ctx->current_cert), buf, 256);
+	X509_NAME_oneline(X509_get_issuer_name(ctx->current_cert), buf, sizeof(buf));
 	syslog(L_NOTICE, "issuer= %s", buf);
 	break;
     case X509_V_ERR_CERT_NOT_YET_VALID:
@@ -286,14 +281,14 @@ static int verify_callback(int ok, X509_STORE_CTX * ctx)
 
 
 /*
- * taken from OpenSSL crypto/bio/b_dump.c, modified to save a lot of strcpy
- * and strcat by Matti Aarnio.
- */
-
+**  Taken from OpenSSL crypto/bio/b_dump.c, modified to save a lot of strcpy
+**  and strcat by Matti Aarnio.
+*/
 #define TRUNCATE
 #define DUMP_WIDTH	16
 
-static int tls_dump(const char *s, int len)
+static int
+tls_dump(const char *s, int len)
 {
     int     ret = 0;
     char    buf[160 + 1];
@@ -317,36 +312,35 @@ static int tls_dump(const char *s, int len)
 	rows++;
 
     for (i = 0; i < rows; i++) {
-	buf[0] = '\0';				/* start with empty string */
+	buf[0] = '\0';				/* Start with empty string. */
 	ss = buf;
 
-	sprintf(ss, "%04x ", i * DUMP_WIDTH);
+	snprintf(ss, sizeof(buf), "%04x ", i * DUMP_WIDTH);
 	ss += strlen(ss);
 	for (j = 0; j < DUMP_WIDTH; j++) {
 	    if (((i * DUMP_WIDTH) + j) >= len) {
-		strcpy(ss, "   ");
+		strlcpy(ss, "   ", sizeof(buf) - (ss - buf));
 	    } else {
 		ch = ((unsigned char) *((const char *)(s) + i * DUMP_WIDTH + j))
 		    & 0xff;
-		sprintf(ss, "%02x%c", ch, j == 7 ? '|' : ' ');
+		snprintf(ss, sizeof(buf) - (ss - buf), "%02x%c", ch,
+                         j == 7 ? '|' : ' ');
 		ss += 3;
 	    }
 	}
 	ss += strlen(ss);
-	*ss+= ' ';
+	*ss += ' ';
 	for (j = 0; j < DUMP_WIDTH; j++) {
 	    if (((i * DUMP_WIDTH) + j) >= len)
 		break;
 	    ch = ((unsigned char) *((const char *)(s) + i * DUMP_WIDTH + j))
 		& 0xff;
-	    *ss+= (((ch >= ' ') && (ch <= '~')) ? ch : '.');
-	    if (j == 7) *ss+= ' ';
+	    *ss += (((ch >= ' ') && (ch <= '~')) ? ch : '.');
+	    if (j == 7) *ss += ' ';
 	}
 	*ss = 0;
-	/* 
-	 * if this is the last call then update the ddt_dump thing so that
-         * we will move the selection point in the debug window
-         */	
+	/* If this is the last call, then update the ddt_dump thing so that
+         * we will move the selection point in the debug window. */
 	if (tls_loglevel>0)
 	  Printf("%s", buf);
 	ret += strlen(buf);
@@ -362,15 +356,16 @@ static int tls_dump(const char *s, int len)
     return (ret);
 }
 
- /*
-  * Set up the cert things on the server side. We do need both the
-  * private key (in key_file) and the cert (in cert_file).
-  * Both files may be identical.
-  *
-  * This function is taken from OpenSSL apps/s_cb.c
-  */
 
-static int set_cert_stuff(SSL_CTX * ctx, char *cert_file, char *key_file)
+/*
+**  Set up the cert things on the server side. We do need both the
+**  private key (in key_file) and the cert (in cert_file).
+**  Both files may be identical.
+**
+**  This function is taken from OpenSSL apps/s_cb.c.
+*/
+static int
+set_cert_stuff(SSL_CTX * ctx, char *cert_file, char *key_file)
 {
     struct stat buf;
 
@@ -383,16 +378,22 @@ static int set_cert_stuff(SSL_CTX * ctx, char *cert_file, char *key_file)
 	if (key_file == NULL)
 	    key_file = cert_file;
 
-	/* check ownership and permissions of key file */
-	if (lstat(key_file, &buf) == -1) {
+	/* Check ownership and permissions of key file.
+         * Look at the real file (stat) and not a possible symlink (lstat). */
+	if (stat(key_file, &buf) == -1) {
 	    syslog(L_ERROR, "unable to stat private key '%s'", key_file);
 	    return (0);
 	}
-	if (!S_ISREG(buf.st_mode) || (buf.st_mode & 0077) != 0 ||
-	    buf.st_uid != getuid()) {
+
+        /* Check that the key file is a real file, not readable by
+         * everyone.  If the mode is 440 or 640, make sure the group owner
+         * is the news group (to prevent the failure case of having news:users
+         * as the owner and group. */
+	if (!S_ISREG(buf.st_mode) || (buf.st_mode & 0137) != 0
+            || ((buf.st_mode & 0040) != 0 && buf.st_gid != getegid())) {
 	    syslog(L_ERROR, "bad ownership or permissions on private key"
-                   " '%s': private key must be mode 600 and owned by "
-                   NEWSUSER, cert_file);
+                   " '%s':  private key must be mode 640 at most, and readable"
+                   " by the news group only", key_file);
 	    return (0);
 	}
 
@@ -402,9 +403,9 @@ static int set_cert_stuff(SSL_CTX * ctx, char *cert_file, char *key_file)
 	    return (0);
 	}
 	/* Now we know that a key and cert have been set against
-         * the SSL context */
+         * the SSL context. */
 	if (!SSL_CTX_check_private_key(ctx)) {
-	    syslog(L_ERROR, "Private key does not match the certificate public key");
+	    syslog(L_ERROR, "private key does not match the certificate public key");
 	    return (0);
 	}
     }
@@ -412,24 +413,19 @@ static int set_cert_stuff(SSL_CTX * ctx, char *cert_file, char *key_file)
 }
 
 
+/*
+**  This is the setup routine for the SSL server.  As nnrpd might be called
+**  more than once, we only want to do the initialization one time.
+**
+**  The skeleton of this function is taken from OpenSSL apps/s_server.c.
+**
+**  Returns -1 on error.
+*/
 
- /*
-  * This is the setup routine for the SSL server. As smtpd might be called
-  * more than once, we only want to do the initialization one time.
-  *
-  * The skeleton of this function is taken from OpenSSL apps/s_server.c.
-
-  * returns -1 on error
-  */
-
-int tls_init_serverengine(int verifydepth,
-			  int askcert,
-			  int requirecert,
-			  char *tls_CAfile,
-			  char *tls_CApath,
-			  char *tls_cert_file,
-			  char *tls_key_file
-			  )
+int
+tls_init_serverengine(int verifydepth, int askcert, int requirecert,
+                      char *tls_CAfile, char *tls_CApath, char *tls_cert_file,
+                      char *tls_key_file)
 {
     int     off = 0;
     int     verify_flags = SSL_VERIFY_NONE;
@@ -440,7 +436,7 @@ int tls_init_serverengine(int verifydepth,
     struct stat buf;
 
     if (tls_serverengine)
-      return (0);				/* already running */
+      return (0);				/* Already running. */
 
     if (tls_loglevel >= 2)
       Printf("starting TLS engine");
@@ -453,7 +449,7 @@ int tls_init_serverengine(int verifydepth,
       return (-1);
     };
 
-    off |= SSL_OP_ALL;		/* Work around all known bugs */
+    off |= SSL_OP_ALL;		/* Work around all known bugs. */
     SSL_CTX_set_options(CTX, off);
     SSL_CTX_set_info_callback(CTX, apps_ssl_info_callback);
     SSL_CTX_sess_set_cache_size(CTX, 128);
@@ -489,8 +485,8 @@ int tls_init_serverengine(int verifydepth,
       return (-1);
     }
 
-    /* load some randomization data from /dev/urandom, if it exists */
-    /* FIXME: should also check for ".rand" file, update it on exit */
+    /* Load some randomization data from /dev/urandom, if it exists.
+     * FIXME:  should also check for ".rand" file, update it on exit. */
     if (stat("/dev/urandom", &buf) == 0)
       RAND_load_file("/dev/urandom", 16 * 1024);
 
@@ -516,68 +512,76 @@ int tls_init_serverengine(int verifydepth,
 **  The function called by nnrpd to initialize the TLS support.  Calls
 **  tls_init_server_engine and checks the result.  On any sort of failure,
 **  nnrpd will exit.
+**
+**  Returns -1 on error.
 */
-void
+int
 tls_init(void)
 {
     int ssl_result;
 
     if (tls_initialized)
-        return;
-    sasl_config_read();
-    ssl_result = tls_init_serverengine(5,        /* depth to verify */
-				       0,        /* can client auth? */
-				       0,        /* required client to auth? */
-				       (char *)sasl_config_getstring("tls_ca_file", ""),
-				       (char *)sasl_config_getstring("tls_ca_path", ""),
-				       (char *)sasl_config_getstring("tls_cert_file", ""),
-				       (char *)sasl_config_getstring("tls_key_file", ""));
+        return 0;
+
+    ssl_result = tls_init_serverengine(5,        /* Depth to verify. */
+				       0,        /* Can client auth? */
+				       0,        /* Required client to auth? */
+				       innconf->tlscafile,
+				       innconf->tlscapath,
+				       innconf->tlscertfile,
+				       innconf->tlskeyfile);
     if (ssl_result == -1) {
-        Reply("%d Error initializing TLS\r\n", NNTP_STARTTLS_BAD_VAL);
+        Reply("%d Error initializing TLS\r\n",
+              initialSSL ? NNTP_FAIL_TERMINATING : NNTP_ERR_STARTTLS);
         syslog(L_ERROR, "error initializing TLS: "
                "[CA_file: %s] [CA_path: %s] [cert_file: %s] [key_file: %s]",
-               sasl_config_getstring("tls_ca_file", ""),
-               sasl_config_getstring("tls_ca_path", ""),
-               sasl_config_getstring("tls_cert_file", ""),
-               sasl_config_getstring("tls_key_file", ""));
-        ExitWithStats(1, false);
+               innconf->tlscafile, innconf->tlscapath,
+               innconf->tlscertfile, innconf->tlskeyfile);
+        if (initialSSL)
+            ExitWithStats(1, false);
+        return -1;
     }
+
     tls_initialized = true;
+    return 0;
 }
 
 
-/* taken from OpenSSL apps/s_cb.c */
-
-static long bio_dump_cb(BIO * bio, int cmd, const char *argp, int argi,
-			long argl UNUSED, long ret)
+/*
+**  Taken from OpenSSL apps/s_cb.c.
+*/
+static long
+bio_dump_cb(BIO * bio, int cmd, const char *argp, int argi, long argl UNUSED,
+            long ret)
 {
     if (!do_dump)
 	return (ret);
 
     if (cmd == (BIO_CB_READ | BIO_CB_RETURN)) {
-	Printf("read from %08X [%08lX] (%d bytes => %ld (0x%X))", (unsigned int) bio, (long unsigned int) argp,
+	Printf("read from %08lX [%08lX] (%d bytes => %ld (0x%X))", (long unsigned int) bio, (long unsigned int) argp,
 		 argi, ret, (unsigned int) ret);
 	tls_dump(argp, (int) ret);
 	return (ret);
     } else if (cmd == (BIO_CB_WRITE | BIO_CB_RETURN)) {
-	Printf("write to %08X [%08lX] (%d bytes => %ld (0x%X))", (unsigned int) bio, (long unsigned int)argp,
+	Printf("write to %08lX [%08lX] (%d bytes => %ld (0x%X))", (long unsigned int) bio, (long unsigned int) argp,
 		 argi, ret, (unsigned int) ret);
 	tls_dump(argp, (int) ret);
     }
     return (ret);
 }
 
- /*
-  * This is the actual startup routine for the connection. We expect
-  * that the buffers are flushed and the "220 Ready to start TLS" was
-  * send to the client, so that we can immediately can start the TLS
-  * handshake process.
-  *
-  * layerbits and authid are filled in on sucess. authid is only
-  * filled in if the client authenticated
-  * 
-  */
-int tls_start_servertls(int readfd, int writefd)
+
+/*
+**  This is the actual startup routine for the connection.  We expect
+**  that the buffers are flushed and the "382 Continue with TLS negociation"
+**  was sent to the client (if using STARTTLS), so that we can immediately
+**  start the TLS handshake process.
+**
+**  layerbits and authid are filled in on success; authid is only
+**  filled in if the client authenticated.
+*/
+int
+tls_start_servertls(int readfd, int writefd)
 {
     int     sts;
     int     keepalive;
@@ -586,7 +590,7 @@ int tls_start_servertls(int readfd, int writefd)
 
     if (!tls_serverengine)
     {		
-      /* should never happen */
+      /* It should never happen. */
       syslog(L_ERROR, "tls_engine not running");
       return (-1);
     }
@@ -603,7 +607,7 @@ int tls_start_servertls(int readfd, int writefd)
     }
     SSL_clear(tls_conn);
 
-#if	defined(SOL_SOCKET) && defined(SO_KEEPALIVE)
+#if defined(SOL_SOCKET) && defined(SO_KEEPALIVE)
     /* Set KEEPALIVE to catch broken socket connections. */
     keepalive = 1;
     if (setsockopt(readfd, SOL_SOCKET, SO_KEEPALIVE, &keepalive, sizeof(keepalive)) < 0)
@@ -612,7 +616,7 @@ int tls_start_servertls(int readfd, int writefd)
         syslog(L_ERROR, "fd %d can't setsockopt(KEEPALIVE) %m", writefd);
 #endif /* SOL_SOCKET && SO_KEEPALIVE */
     
-    /* set the file descriptors for SSL to use */
+    /* Set the file descriptors for SSL to use. */
     if (SSL_set_rfd(tls_conn, readfd)==0)
     {
       return (-1);
@@ -623,25 +627,21 @@ int tls_start_servertls(int readfd, int writefd)
       return (-1);
     }
     
-    /*
-     * This is the actual handshake routine. It will do all the negotiations
-     * and will check the client cert etc.
-     */
+    /* This is the actual handshake routine.  It will do all the negotiations
+     * and will check the client cert etc. */
     SSL_set_accept_state(tls_conn);
 
-    /*
-     * We do have an SSL_set_fd() and now suddenly a BIO_ routine is called?
+    /* We do have an SSL_set_fd() and now suddenly a BIO_ routine is called?
      * Well there is a BIO below the SSL routines that is automatically
-     * created for us, so we can use it for debugging purposes.
-     */
+     * created for us, so we can use it for debugging purposes. */
     if (tls_loglevel >= 3)
 	BIO_set_callback(SSL_get_rbio(tls_conn), bio_dump_cb);
 
-    /* Dump the negotiation for loglevels 3 and 4*/
+    /* Dump the negotiation for loglevels 3 and 4. */
     if (tls_loglevel >= 3)
 	do_dump = 1;
 
-      if ((sts = SSL_accept(tls_conn)) <= 0) { /* xxx <= 0 */
+      if ((sts = SSL_accept(tls_conn)) <= 0) {
 	session = SSL_get_session(tls_conn);
 
 	if (session) {
@@ -652,12 +652,12 @@ int tls_start_servertls(int readfd, int writefd)
 	tls_conn = NULL;
 	return (-1);
       }
-      /* Only loglevel==4 dumps everything */
+      /* Only loglevel 4 dumps everything. */
       if (tls_loglevel < 4)
 	do_dump = 0;
 
     tls_protocol = SSL_get_version(tls_conn);
-    cipher = SSL_get_current_cipher(tls_conn);
+    cipher = (SSL_CIPHER *) SSL_get_current_cipher(tls_conn);
 
     tls_cipher_name = SSL_CIPHER_get_name(cipher);
     tls_cipher_usebits = SSL_CIPHER_get_bits(cipher,
@@ -670,22 +670,20 @@ int tls_start_servertls(int readfd, int writefd)
     return (0);
 }
 
+
 ssize_t
-SSL_writev (ssl, vector, count)
-     SSL *ssl;
-     const struct iovec *vector;
-     int count;
+SSL_writev (SSL *ssl, const struct iovec *vector, int count)
 {
   static char *buffer = NULL;
   static size_t allocsize = 0;
   char *bp;
   size_t bytes, to_copy;
   int i;
-  /* Find the total number of bytes to be written.  */
+  /* Find the total number of bytes to be written. */
   bytes = 0;
   for (i = 0; i < count; ++i)
     bytes += vector[i].iov_len;
-  /* Allocate a buffer to hold the data.  */
+  /* Allocate a buffer to hold the data. */
   if (NULL == buffer) {
     buffer = (char *) xmalloc(bytes);
     allocsize = bytes;
@@ -693,7 +691,7 @@ SSL_writev (ssl, vector, count)
     buffer = (char *) xrealloc (buffer, bytes);
     allocsize = bytes;
   }
-  /* Copy the data into BUFFER.  */
+  /* Copy the data into BUFFER. */
   to_copy = bytes;
   bp = buffer;
   for (i = 0; i < count; ++i)
@@ -709,5 +707,4 @@ SSL_writev (ssl, vector, count)
   return SSL_write (ssl, buffer, bytes);
 }
 
-
 #endif /* HAVE_SSL */
