@@ -1,4 +1,4 @@
-/*  $Id: trash.c 8618 2009-09-07 08:23:47Z iulius $
+/*  $Id: trash.c 8818 2009-11-17 18:58:24Z iulius $
 **
 **  Storage manager module for trash method.
 */
@@ -82,6 +82,16 @@ trash_printfiles(FILE *file UNUSED, TOKEN token UNUSED, char **xref UNUSED,
 {
 }
 
+char *
+trash_explaintoken(const TOKEN token UNUSED)
+{
+    char *text;
+
+    xasprintf(&text, "method=%s", "trash");
+
+    return text;
+}
+
 ARTHANDLE *
 trash_next(ARTHANDLE *article UNUSED, const RETRTYPE amount UNUSED)
 {
